@@ -15,9 +15,12 @@
 # Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
 # trademarks that are incorporated in the source code or documentation are not
 # subject to the GNU General Public License and may only be used or replicated
-# with the express permission of Red Hat, Inc. 
+# with the express permission of Red Hat, Inc.
 #
-from pykickstart.base import *
+from pykickstart.version import FC3, F29, versionToLongString
+from pykickstart.base import KickstartCommand, DeprecatedCommand
+from pykickstart.options import KSOptionParser
+
 
 class FC3_DeviceProbe(KickstartCommand):
     removedKeywords = KickstartCommand.removedKeywords
@@ -30,7 +33,7 @@ class FC3_DeviceProbe(KickstartCommand):
     def __str__(self):
         retval = KickstartCommand.__str__(self)
 
-        if self.deviceprobe != "":
+        if self.deviceprobe:
             retval += "deviceprobe %s\n" % self.deviceprobe
 
         return retval
@@ -38,3 +41,17 @@ class FC3_DeviceProbe(KickstartCommand):
     def parse(self, args):
         self.deviceprobe = " ".join(args)
         return self
+
+    def _getParser(self):
+        return KSOptionParser(prog="deviceprobe", version=FC3, description="probe for devices")
+
+
+class F29_DeviceProbe(DeprecatedCommand, FC3_DeviceProbe):
+
+    def __init__(self):  # pylint: disable=super-init-not-called
+        DeprecatedCommand.__init__(self)
+
+    def _getParser(self):
+        op = FC3_DeviceProbe._getParser(self)
+        op.description += "\n\n.. deprecated:: %s" % versionToLongString(F29)
+        return op
