@@ -15,13 +15,13 @@
 # Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
 # trademarks that are incorporated in the source code or documentation are not
 # subject to the GNU General Public License and may only be used or replicated
-# with the express permission of Red Hat, Inc. 
+# with the express permission of Red Hat, Inc.
 #
-from pykickstart.base import *
-from pykickstart.options import *
-
-import gettext
-_ = lambda x: gettext.ldgettext("pykickstart", x)
+from pykickstart.version import FC3, F8, RHEL6, F29
+from pykickstart.base import KickstartCommand
+from pykickstart.errors import KickstartParseError
+from pykickstart.i18n import _
+from pykickstart.options import KSOptionParser, commaSplit
 
 class FC3_IgnoreDisk(KickstartCommand):
     removedKeywords = KickstartCommand.removedKeywords
@@ -36,24 +36,25 @@ class FC3_IgnoreDisk(KickstartCommand):
     def __str__(self):
         retval = KickstartCommand.__str__(self)
 
-        if len(self.ignoredisk) > 0:
+        if self.ignoredisk:
             retval += "ignoredisk --drives=%s\n" % ",".join(self.ignoredisk)
 
         return retval
 
     def _getParser(self):
-        def drive_cb (option, opt_str, value, parser):
-            for d in value.split(','):
-                parser.values.ensure_value(option.dest, []).append(d)
-
-        op = KSOptionParser()
-        op.add_option("--drives", dest="ignoredisk", action="callback",
-                      callback=drive_cb, nargs=1, type="string", required=1)
+        op = KSOptionParser(prog="ignoredisk", description="""
+            Controls anaconda's access to disks attached to the system. By
+            default, all disks will be available for partitioning. Only one of
+            the following three options may be used.""", version=FC3)
+        op.add_argument("--drives", dest="ignoredisk", type=commaSplit,
+                        required=True, version=FC3, help="""
+                        Specifies those disks that anaconda should not touch
+                        when partitioning, formatting, and clearing.""")
         return op
 
     def parse(self, args):
-        (opts, extra) = self.op.parse_args(args=args, lineno=self.lineno)
-        self._setToSelf(self.op, opts)
+        ns = self.op.parse_args(args=args, lineno=self.lineno)
+        self.set_to_self(ns)
         return self
 
 class F8_IgnoreDisk(FC3_IgnoreDisk):
@@ -68,32 +69,36 @@ class F8_IgnoreDisk(FC3_IgnoreDisk):
     def __str__(self):
         retval = KickstartCommand.__str__(self)
 
-        if len(self.ignoredisk) > 0:
+        if self.ignoredisk:
             retval += "ignoredisk --drives=%s\n" % ",".join(self.ignoredisk)
-        elif len(self.onlyuse) > 0:
+        elif self.onlyuse:
             retval += "ignoredisk --only-use=%s\n" % ",".join(self.onlyuse)
 
         return retval
 
-    def parse(self, args, errorCheck=True):
+    def parse(self, args):
         retval = FC3_IgnoreDisk.parse(self, args)
 
-        if errorCheck:
-            if (len(self.ignoredisk) == 0 and len(self.onlyuse) == 0) or (len(self.ignoredisk) > 0 and (len(self.onlyuse) > 0)):
-                raise KickstartValueError, formatErrorMsg(self.lineno, msg=_("One of --drives or --only-use must be specified for ignoredisk command."))
+        howmany = 0
+        if self.ignoredisk:
+            howmany += 1
+        if self.onlyuse:
+            howmany += 1
+        if howmany != 1:
+            raise KickstartParseError(_("One of --drives or --only-use must be specified "
+                                        "for ignoredisk command."), lineno=self.lineno)
 
         return retval
 
     def _getParser(self):
-        def drive_cb (option, opt_str, value, parser):
-            for d in value.split(','):
-                parser.values.ensure_value(option.dest, []).append(d)
-
         op = FC3_IgnoreDisk._getParser(self)
-        op.add_option("--drives", dest="ignoredisk", action="callback",
-                      callback=drive_cb, nargs=1, type="string")
-        op.add_option("--only-use", dest="onlyuse", action="callback",
-                      callback=drive_cb, nargs=1, type="string")
+        op.add_argument("--drives", dest="ignoredisk", version=F8,
+                        type=commaSplit, help="""
+                        This argument is no longer required!""")
+        op.add_argument("--only-use", dest="onlyuse",
+                        type=commaSplit, version=F8, help="""
+                        Specifies the opposite - only disks listed here will be
+                        used during installation.""")
         return op
 
 class RHEL6_IgnoreDisk(F8_IgnoreDisk):
@@ -102,10 +107,7 @@ class RHEL6_IgnoreDisk(F8_IgnoreDisk):
 
     def __init__(self, writePriority=0, *args, **kwargs):
         F8_IgnoreDisk.__init__(self, writePriority, *args, **kwargs)
-
         self.interactive = kwargs.get("interactive", False)
-        if self.interactive:
-            self.ignoredisk = []
 
     def __str__(self):
         retval = F8_IgnoreDisk.__str__(self)
@@ -116,24 +118,45 @@ class RHEL6_IgnoreDisk(F8_IgnoreDisk):
         return retval
 
     def parse(self, args):
-        retval = F8_IgnoreDisk.parse(self, args, errorCheck=False)
+        retval = FC3_IgnoreDisk.parse(self, args)
 
         howmany = 0
-        if len(self.ignoredisk) > 0:
+        if self.ignoredisk:
             howmany += 1
-        if len(self.onlyuse) > 0:
+        if self.onlyuse:
             howmany += 1
         if self.interactive:
             howmany += 1
         if howmany != 1:
-            raise KickstartValueError, formatErrorMsg(self.lineno, msg=_("One of --drives , --only-use , or --interactive must be specified for ignoredisk command."))
+            raise KickstartParseError(_("One of --drives , --only-use , or --interactive must be specified for ignoredisk command."), lineno=self.lineno)
+
+        if self.interactive:
+            self.ignoredisk = []
 
         return retval
 
     def _getParser(self):
         op = F8_IgnoreDisk._getParser(self)
-        op.add_option("--interactive", dest="interactive", action="store_true",
-                      default=False)
+        op.add_argument("--interactive", action="store_true",
+                        default=False, version=RHEL6, help="""
+                        Allow the user manually navigate the advanced storage
+                        screen.""")
         return op
 
-F14_IgnoreDisk = RHEL6_IgnoreDisk
+class F14_IgnoreDisk(RHEL6_IgnoreDisk):
+    pass
+
+class F29_IgnoreDisk(F14_IgnoreDisk):
+    removedKeywords = F14_IgnoreDisk.removedKeywords
+    removedAttrs = F14_IgnoreDisk.removedAttrs
+
+    def parse(self, args):
+        return F8_IgnoreDisk.parse(self, args)
+
+    def _getParser(self):
+        op = F14_IgnoreDisk._getParser(self)
+        op.add_argument("--interactive", action="store_true",
+                        default=False, deprecated=F29, help="""
+                        Allow the user manually navigate the advanced storage
+                        screen.""")
+        return op
