@@ -43,7 +43,6 @@ import six
 import shlex
 import sys
 import warnings
-from ordered_set import OrderedSet
 
 from pykickstart import constants, version
 from pykickstart.errors import KickstartError, KickstartParseError, KickstartParseWarning
@@ -182,7 +181,7 @@ class PutBackIterator(Iterator):
             return next(self._iterable)
 
     def __next__(self):
-        return self.next()
+        return self.next()                          # pylint: disable=not-callable
 
 ###
 ### SCRIPT HANDLING
@@ -337,6 +336,9 @@ class Packages(KickstartObject):
            handleMissing -- If unknown packages are specified in the %packages
                             section, should it be ignored or not?  Values can
                             be KS_MISSING_* from pykickstart.constants.
+           handleBroken  -- If packages with conflicts are specified in the
+                            %packages section, should it be ignored or not?
+                            Values can be KS_BROKEN_* from pykickstart.constants.
            packageList   -- A list of all the packages specified in the
                             %packages section.
            instLangs     -- A list of languages to install.
@@ -361,6 +363,7 @@ class Packages(KickstartObject):
         self.excludeDocs = False
         self.groupList = []
         self.handleMissing = constants.KS_MISSING_PROMPT
+        self.handleBroken = constants.KS_BROKEN_REPORT
         self.packageList = []
         self.instLangs = None
         self.multiLib = False
@@ -371,36 +374,8 @@ class Packages(KickstartObject):
 
     def __str__(self):
         """Return a string formatted for output to a kickstart file."""
-        pkgs = ""
-
-        if not self.default:
-            if self.environment:
-                pkgs += "@^%s\n" % self.environment
-
-            grps = self.groupList
-            grps.sort()
-            for grp in grps:
-                pkgs += "%s\n" % grp.__str__()
-
-            p = self.packageList
-            p.sort()
-            for pkg in p:
-                pkgs += "%s\n" % pkg
-
-            grps = self.excludedGroupList
-            grps.sort()
-            for grp in grps:
-                pkgs += "-%s\n" % grp.__str__()
-
-            p = self.excludedList
-            p.sort()
-            for pkg in p:
-                pkgs += "-%s\n" % pkg
-
-            if pkgs == "" and not self.seen:
-                return ""
-
-        retval = "\n%packages"
+        pkgs = self._processPackagesContent()
+        retval = ""
 
         if self.default:
             retval += " --default"
@@ -412,21 +387,55 @@ class Packages(KickstartObject):
             retval += " --nocore"
         if self.handleMissing == constants.KS_MISSING_IGNORE:
             retval += " --ignoremissing"
+        if self.handleBroken == constants.KS_BROKEN_IGNORE:
+            retval += " --ignorebroken"
         if self.instLangs is not None:
-            retval += " --instLangs=%s" % self.instLangs
+            retval += " --inst-langs=%s" % self.instLangs
         if self.multiLib:
             retval += " --multilib"
         if self.excludeWeakdeps:
-            retval += " --excludeWeakdeps"
+            retval += " --exclude-weakdeps"
         if self.timeout is not None:
             retval += " --timeout=%d" % self.timeout
         if self.retries is not None:
             retval += " --retries=%d" % self.retries
 
+        if retval == "" and pkgs == "" and not self.seen:
+            return ""
+
         if self._ver >= version.F8:
-            return retval + "\n" + pkgs + "\n%end\n"
+            return "\n%packages" + retval + "\n" + pkgs + "\n%end\n"
         else:
-            return retval + "\n" + pkgs + "\n"
+            return "\n%packages" + retval + "\n" + pkgs + "\n"
+
+    def _processPackagesContent(self):
+        pkgs = ""
+
+        if not self.default:
+            if self.environment:
+                pkgs += "@^%s\n" % self.environment
+
+        grps = self.groupList
+        grps.sort()
+        for grp in grps:
+            pkgs += "%s\n" % grp.__str__()
+
+        p = self.packageList
+        p.sort()
+        for pkg in p:
+            pkgs += "%s\n" % pkg
+
+        grps = self.excludedGroupList
+        grps.sort()
+        for grp in grps:
+            pkgs += "-%s\n" % grp.__str__()
+
+        p = self.excludedList
+        p.sort()
+        for pkg in p:
+            pkgs += "-%s\n" % pkg
+
+        return pkgs
 
     def _processGroup(self, line):
         op = KSOptionParser(prog="", description="", version=version.DEVEL)
@@ -458,10 +467,10 @@ class Packages(KickstartObject):
         """Given a list of lines from the input file, strip off any leading
            symbols and add the result to the appropriate list.
         """
-        existingExcludedSet = OrderedSet(self.excludedList)
-        existingPackageSet = OrderedSet(self.packageList)
-        newExcludedSet = OrderedSet()
-        newPackageSet = OrderedSet()
+        existingExcludedSet = set(self.excludedList)
+        existingPackageSet = set(self.packageList)
+        newExcludedSet = set()
+        newPackageSet = set()
 
         excludedGroupList = []
 
@@ -501,8 +510,8 @@ class Packages(KickstartObject):
         existingExcludedSet = (existingExcludedSet - existingPackageSet) | newExcludedSet
 
         # FIXME: figure these types out
-        self.packageList = list(existingPackageSet)
-        self.excludedList = list(existingExcludedSet)
+        self.packageList = sorted(existingPackageSet)
+        self.excludedList = sorted(existingExcludedSet)
 
 ###
 ### PARSER
