@@ -29,12 +29,14 @@ treated just the same as a predefined one by the kickstart parser.  All that
 is necessary is to create a new subclass of Section and call
 parser.registerSection with an instance of your new class.
 """
+import warnings
 from pykickstart.constants import KS_SCRIPT_PRE, KS_SCRIPT_POST, KS_SCRIPT_TRACEBACK, \
                                   KS_SCRIPT_PREINSTALL, KS_SCRIPT_ONERROR, \
-                                  KS_MISSING_IGNORE, KS_MISSING_PROMPT
-from pykickstart.errors import KickstartParseError
+                                  KS_MISSING_IGNORE, KS_MISSING_PROMPT, \
+                                  KS_BROKEN_IGNORE, KS_BROKEN_REPORT
+from pykickstart.errors import KickstartParseError, KickstartDeprecationWarning
 from pykickstart.options import KSOptionParser
-from pykickstart.version import FC4, F7, F9, F18, F21, F22, F24, RHEL6, RHEL7
+from pykickstart.version import FC4, F7, F9, F18, F21, F22, F24, F32, RHEL6, RHEL7
 from pykickstart.i18n import _
 
 class Section(object):
@@ -74,7 +76,6 @@ class Section(object):
         """This method is called when the %end tag for a section is seen.  It
            is not required to be provided.
         """
-        pass
 
     def handleLine(self, line):
         """This method is called for every line of a section.  Take whatever
@@ -85,7 +86,6 @@ class Section(object):
 
            line -- The complete line, with any trailing newline.
         """
-        pass
 
     # pylint: disable=unused-argument
     def handleHeader(self, lineno, args):
@@ -454,7 +454,8 @@ class PackageSection(Section):
                             Packages can be specified by group or by individual
                             package name. The installation program defines
                             several groups that contain related packages. Refer
-                            to the repodata/\\*comps\\*.xml file on the first CD-ROM
+                            to the repodata/\\*comps\\*.xml file on the
+                            installation media or in installation repository
                             for a list of groups. Each group has an id, user
                             visibility value, name, description, and package
                             list. In the package list, the packages marked as
@@ -588,8 +589,8 @@ class PackageSection(Section):
                         version=FC4)
         op.add_argument("--ignoremissing", action="store_true", default=False,
                         help="""
-                        Ignore any packages or groups specified in the packages
-                        section that are not found in any configured repository.
+                        Ignore any packages, groups or module streams specified in the
+                        packages section that are not found in any configured repository.
                         The default behavior is to halt the installation and ask
                         the user if the installation should be aborted or
                         continued. This option allows fully automated
@@ -605,7 +606,7 @@ class PackageSection(Section):
 
         op.add_argument("--default", dest="defaultPackages", action="store_true",
                         default=False, version=F7, help="""
-                        Install the default package set. This corresponds to the
+                        Install the default environment. This corresponds to the
                         package set that would be installed if no other
                         selections were made on the package customization screen
                         during an interactive install.""")
@@ -615,7 +616,7 @@ class PackageSection(Section):
 
         op.remove_argument("--ignoredeps", version=F9)
         op.remove_argument("--resolvedeps", version=F9)
-        op.add_argument("--instLangs", default=None, version=F9, help="""
+        op.add_argument("--instLangs", "--inst-langs", default=None, version=F9, help="""
                         Specify the list of languages that should be installed.
                         This is different from the package group level
                         selections, though. This option does not specify what
@@ -674,7 +675,7 @@ class PackageSection(Section):
         if self.version < F24:
             return op
 
-        op.add_argument("--excludeWeakdeps", dest="excludeWeakdeps",
+        op.add_argument("--excludeWeakdeps", "--exclude-weakdeps", dest="excludeWeakdeps",
                         action="store_true", default=False, version=F24,
                         help="""
                         Do not install packages from weak dependencies. These
@@ -682,6 +683,21 @@ class PackageSection(Section):
                         Recommends and Supplements flags. By default weak
                         dependencies will be installed.""")
 
+        if self.version < F32:
+            return op
+
+        op.add_argument("--ignorebroken", action="store_true", default=False, version=F32,
+                        help="""
+                        Ignore any packages, groups or modules with conflicting files.
+                        This issue will disable the DNF `strict` option. The default behavior
+                        is to abort the installation with error message describing the
+                        conflicting files.
+
+                        **WARNING: Usage of this parameter is DISCOURAGED! The DNF
+                        strict option will NOT log any information about what packages
+                        were skipped. Using this option may result in an unusable system.**
+                        """)
+
         return op
 
     def handleHeader(self, lineno, args):
@@ -743,3 +759,20 @@ class PackageSection(Section):
             return
 
         self.handler.packages.excludeWeakdeps = ns.excludeWeakdeps
+
+        if self.version < F32:
+            return
+
+        if ns.ignorebroken:
+            self.handler.packages.handleBroken = KS_BROKEN_IGNORE
+        else:
+            self.handler.packages.handleBroken = KS_BROKEN_REPORT
+
+        for option, new_option in \
+                {"--instLangs": "--inst-langs", "--excludeWeakdeps": "--exclude-weakdeps"}.items():
+            if option in args:
+                warnings.warn(_("The %(option)s option on line %(lineno)s will be deprecated in"
+                                "future releases. Please modify your kickstart file to replace"
+                                "this option with its preferred alias %(new_option)s.")
+                              % {"option": option, "lineno": lineno, "new_option": new_option},
+                              KickstartDeprecationWarning)
