@@ -1,7 +1,7 @@
 #
 # Chris Lumens <clumens@redhat.com>
 #
-# Copyright 2009 Red Hat, Inc.
+# Copyright 2009-2014 Red Hat, Inc.
 #
 # This copyrighted material is made available to anyone wishing to use, modify,
 # copy, or redistribute it subject to the terms and conditions of the GNU
@@ -15,13 +15,11 @@
 # Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
 # trademarks that are incorporated in the source code or documentation are not
 # subject to the GNU General Public License and may only be used or replicated
-# with the express permission of Red Hat, Inc. 
+# with the express permission of Red Hat, Inc.
 #
-import unittest, shlex
-import warnings
-from tests.baseclass import *
-
-from pykickstart.errors import *
+import unittest
+from tests.baseclass import CommandTest
+from pykickstart.errors import KickstartDeprecationWarning
 
 class FC3_TestCase(CommandTest):
     command = "bootloader"
@@ -33,7 +31,7 @@ class FC3_TestCase(CommandTest):
             linear = ""
 
         # pass
-        self.assert_parse("bootloader --append=rhgb","bootloader --append=\"rhgb\" %s--location=mbr\n" % linear)
+        self.assert_parse("bootloader --append=rhgb", "bootloader --append=\"rhgb\" %s--location=mbr\n" % linear)
         self.assert_parse("bootloader --append=\"rhgb quiet\"", "bootloader --append=\"rhgb quiet\" %s--location=mbr\n" % linear)
         self.assert_parse("bootloader", "bootloader %s--location=mbr\n" % linear)
 
@@ -56,13 +54,21 @@ class FC3_TestCase(CommandTest):
 
         if "--useLilo" in self.optionList:
             self.assert_parse("bootloader --useLilo", "bootloader %s--location=mbr --useLilo\n" % linear)
+            self.assert_parse("lilo")
 
         # fail
-        self.assert_parse_error("bootloader --append", KickstartParseError)
-        self.assert_parse_error("bootloader --location=nowhere", KickstartParseError)
-        self.assert_parse_error("bootloader --password", KickstartParseError)
-        self.assert_parse_error("bootloader --md5pass", KickstartParseError)
-        self.assert_parse_error("bootloader --driveorder", KickstartParseError)
+        self.assert_parse_error("bootloader --append")
+        self.assert_parse_error("bootloader --location=nowhere")
+        self.assert_parse_error("bootloader --password")
+        self.assert_parse_error("bootloader --md5pass")
+        self.assert_parse_error("bootloader --driveorder")
+
+        # extra test coverage
+        cmd = self.handler().commands[self.command]
+        cmd.location = ""
+        cmd.linear = False
+        self.assertEqual(cmd.__str__(), "")
+        self.assertEqual(cmd._getArgsAsStr(), "")
 
 class FC4_TestCase(FC3_TestCase):
     def runTest(self, iscrypted=False):
@@ -84,8 +90,8 @@ class F8_TestCase(FC4_TestCase):
         self.assert_parse("bootloader --default=this", "bootloader --location=mbr --default=this\n")
 
         # fail
-        self.assert_parse_error("bootloader --timeout", KickstartParseError)
-        self.assert_parse_error("bootloader --default", KickstartParseError)
+        self.assert_parse_error("bootloader --timeout")
+        self.assert_parse_error("bootloader --default")
 
 class F12_TestCase(F8_TestCase):
     def runTest(self, iscrypted=False):
@@ -101,7 +107,7 @@ class F14_TestCase(F12_TestCase):
         F12_TestCase.runTest(self, iscrypted)
 
         # fail
-        self.assert_parse_error("bootloader --lba32", KickstartParseError)
+        self.assert_parse_error("bootloader --lba32")
 
 class F15_TestCase(F14_TestCase):
     def runTest(self, iscrypted=False):
@@ -112,6 +118,29 @@ class F15_TestCase(F14_TestCase):
         self.assert_parse("bootloader --password=blahblah --iscrypted", "bootloader --location=mbr --password=\"blahblah\" --iscrypted\n")
         self.assert_parse("bootloader --md5pass=blahblah", "bootloader --location=mbr --password=\"blahblah\" --iscrypted\n")
 
+class F17_TestCase(F15_TestCase):
+    def runTest(self, iscrypted=False):
+        # run parent tests
+        F15_TestCase.runTest(self, iscrypted=iscrypted)
+
+        self.assert_parse("bootloader --location=mbr --boot-drive=/dev/sda",
+                          "bootloader --location=mbr --boot-drive=/dev/sda\n")
+        self.assert_parse("bootloader --location=mbr --boot-drive=sda",
+                          "bootloader --location=mbr --boot-drive=sda\n")
+        self.assert_parse("bootloader --location=mbr --boot-drive=/dev/disk/by-path/pci-0000:00:0e.0-scsi-0:0:0:0",
+                          "bootloader --location=mbr --boot-drive=/dev/disk/by-path/pci-0000:00:0e.0-scsi-0:0:0:0\n")
+
+        self.assert_parse_error("bootloader --location=mbr --boot-drive=sda,sdb")
+
+class F18_TestCase(F17_TestCase):
+    def runTest(self, iscrypted=False):
+        # run parent tests
+        F17_TestCase.runTest(self, iscrypted=iscrypted)
+
+        self.assert_parse("bootloader --location=mbr --timeout=5 --append=\"rhgb quiet\"")
+        self.assert_parse("bootloader --location=mbr --timeout=5 --leavebootorder --append=\"rhgb quiet\"",
+                          "bootloader --append=\"rhgb quiet\" --location=mbr --timeout=5 --leavebootorder\n")
+
 class RHEL5_TestCase(FC4_TestCase):
     def runTest(self, iscrypted=False):
         FC4_TestCase.runTest(self, iscrypted)
@@ -120,7 +149,7 @@ class RHEL5_TestCase(FC4_TestCase):
                           "bootloader --location=mbr --hvargs=\"bleh\"\n")
         self.assert_parse("bootloader --hvargs=\"bleh bleh\"",
                           "bootloader --location=mbr --hvargs=\"bleh bleh\"\n")
-        self.assert_parse_error("bootloader --hvargs", KickstartParseError)
+        self.assert_parse_error("bootloader --hvargs")
 
 class RHEL6_TestCase(F12_TestCase):
     def runTest(self, iscrypted=False):
@@ -131,5 +160,39 @@ class RHEL6_TestCase(F12_TestCase):
         self.assert_parse("bootloader --password=blahblah --iscrypted", "bootloader --location=mbr --password=\"blahblah\" --iscrypted\n")
         self.assert_parse("bootloader --md5pass=blahblah", "bootloader --location=mbr --password=\"blahblah\" --iscrypted\n")
 
+class F19_TestCase(F18_TestCase):
+    def runTest(self, iscrypted=False):
+        # run parent tests
+        F18_TestCase.runTest(self, iscrypted=iscrypted)
+
+        self.assert_parse("bootloader --location=mbr --timeout=5 --append=\"rhgb quiet\"")
+        self.assert_parse("bootloader --location=mbr --timeout=5 --extlinux --append=\"rhgb quiet\"",
+                          "bootloader --append=\"rhgb quiet\" --location=mbr --timeout=5 --extlinux\n")
+
+class F21_TestCase(F19_TestCase):
+    def runTest(self, iscrypted=False):
+        # run parent tests
+        F19_TestCase.runTest(self, iscrypted=iscrypted)
+
+        self.assert_parse("bootloader --disabled", "bootloader --disabled\n")
+        self.assert_parse("bootloader --location=mbr --disabled", "bootloader --disabled\n")
+        self.assert_parse("bootloader --location=mbr --nombr", "bootloader --location=mbr --nombr\n")
+
+class RHEL7_TestCase(F21_TestCase):
+    def runTest(self, iscrypted=False):
+        F21_TestCase.runTest(self, iscrypted=iscrypted)
+
+class F29_TestCase(F21_TestCase):
+    def runTest(self, iscrypted=False):
+        F21_TestCase.runTest(self, iscrypted=iscrypted)
+        self.assert_deprecated("bootloader", "--upgrade")
+
+        # deprecated options should also raise a deprecation warning - test that somewhere
+        with self.assertWarns(KickstartDeprecationWarning):
+            self.assert_parse("bootloader --upgrade")
+
+class RHEL8_TestCase(F29_TestCase):
+    pass
+
 if __name__ == "__main__":
     unittest.main()
