@@ -9,9 +9,8 @@ class PykickstartLintConfig(PocketLintConfig):
         PocketLintConfig.__init__(self)
 
         self.falsePositives = [
-            FalsePositive(r"^E1102.*: .*_TestCase.runTest: self.handler is not callable$"),
             FalsePositive(r"^W1113.*: Keyword argument before variable positional arguments list in the definition of __init__ function$"),
-            FalsePositive(r"^W0107.*: Unnecessary pass statement$"),
+            FalsePositive("W0707.*raise-missing-from"),
         ]
 
     @property
@@ -22,4 +21,7 @@ if __name__ == "__main__":
     conf = PykickstartLintConfig()
     linter = PocketLinter(conf)
     rc = linter.run()
-    sys.exit(rc)
+    if rc in [0, 4]:
+        sys.exit(0)
+    else:
+        sys.exit(rc)
